/* 
	ctrlproxy: A modular IRC proxy
	(c) 2002 Jelmer Vernooij <jelmer@nl.linux.org>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CTRLPROXY_H__
#define __CTRLPROXY_H__

#define IRC_MSG_LEN 540
#define MAXHOSTNAMELEN 4096
#define CTRLPROXY_VERSION "2.2"

enum hook_types { HOOKS_LOGIN = 0 };

#include <unistd.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdarg.h>
#include <glib.h>
#include <gmodule.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <sys/select.h>

struct server;
struct client_data;
struct line;

enum data_direction { UNKNOWN = 0, TO_SERVER = 1, FROM_SERVER = 2 };

struct line {
	char initialized;
	enum data_direction direction;
	struct server *server;
	struct client_data *client;
	char *origin;
	char **args; /* NULL terminated */
	size_t argc;
};

struct nick {
	char *name;
	char mode;
	size_t id;
};

struct channel {
	char *name;
	char *topic;
	char mode;
	char introduced;
	char namreply_started;
	size_t id;
	GList *nicks;
};

struct client_data {
	struct server *server;
	char authenticated;
	GIOChannel *iochannel;
	time_t connect_time;
};

struct server {
	xmlNodePtr xmlConf;
	char *name;
	char *username;
	char *fullname;
	char *hostmask;
	char *password;
	char *nick;
	GIOChannel *iochannel;
	GIOChannel *listen_socket;
	GList *channels;
	char authenticated;
	struct client_data client_data[FD_SETSIZE];
	GList *clients;
};

typedef int (*connect_function) (struct server *);
typedef void (*plugin_init_function) (GModule *, xmlNodePtr);
extern GHashTable *connection_types;
extern GHookList data_hook; 
extern void (*replicate_function) (struct server *, GIOChannel *);

#define CTRLPROXY_INTERFACE_VERSION 5
#define CTRLPROXY_VERSIONING_MAGIC int ctrlproxy_module_version = CTRLPROXY_INTERFACE_VERSION;

/* state.c */
struct channel *find_channel(struct server *st, char *name);
struct nick *find_nick(struct channel *c, char *name);
GSList *gen_replication(struct server *s);

/* server.c */
struct server *find_server(char *name);
extern char my_hostname[MAXHOSTNAMELEN+2];
int server_send_raw(struct server *s, char *data);
void default_replicate_function (struct server *, GIOChannel *);
extern GList *servers;

/* util.c */
void init_sockaddr (struct sockaddr_in *name, const char *hostname, uint16_t port);
int make_socket (uint16_t port);

/* line.c */
struct line *linedup(struct line *l);
struct line * irc_parse_line(char *data);
struct line * virc_parse_line(char *origin, va_list ap);
char *irc_line_string(struct line *l);
char *irc_line_string_nl(struct line *l);
char *line_get_nick(struct line *l);
void free_line(struct line *l);
gboolean irc_send_args(GIOChannel *c, ...);
gboolean irc_sendf(GIOChannel *c, char *fmt, ...);
int irc_send_line(GIOChannel *c, struct line *l);
GIOStatus irc_recv_line(GIOChannel *io, struct line **l, GError **error);
struct line *irc_parse_linef( char *origin, ... );
struct line *irc_parse_line_args( char *origin, ... );

/* main.c */
void log_handler(const gchar *log_domain, GLogLevelFlags flags, const gchar *message, gpointer user_data);
#define add_log_domain(domain) g_log_set_handler (domain, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, log_handler, NULL);

/* Addition to GLib */
GList *g_list_replace(GList *l, gpointer data);

#endif /* __CTRLPROXY_H__ */
